package com.foundeo.spellchecker;

import com.swabunga.spell.event.SpellCheckListener;
import com.swabunga.spell.event.SpellChecker;
import com.swabunga.spell.engine.SpellDictionaryHashMap;
import com.swabunga.spell.engine.Configuration;
import com.swabunga.spell.event.SpellCheckEvent;
import com.swabunga.spell.event.StringWordTokenizer;
import com.swabunga.spell.engine.Word;
import java.io.File;
import java.util.Iterator;

public class Checker implements SpellCheckListener {
	private static Object lock = null;
	static {
		System.setProperty("jazzy.config", "com.foundeo.spellchecker.DynamicConfiguration");
		lock = new Object();
	}

	SpellChecker spellChecker = new SpellChecker();

	/** the max number of characters to spell check */
	protected int charlimit = 24000;

    /** the number of suggestions the speller will return */
    protected int numSuggestions = 14;

    /** how deep to search */
    protected int searchDepth = 150;

    protected boolean ignoreTrailingS = true;

	/** String encoding. default=ISO8859_1 */
	protected String characterSet = "ISO-8859-1";

	protected MisspelledCollection misspelled = new MisspelledCollection();

	private int minWordLength = 2;

	private static final boolean isEvaluationMode = false;



    public Checker() {
    	spellChecker.addSpellCheckListener(this);
    	spellChecker.getConfiguration().setBoolean(Configuration.SPELL_IGNOREINTERNETADDRESSES, true);
    	spellChecker.getConfiguration().setBoolean(Configuration.SPELL_IGNOREDIGITWORDS, true);
    	spellChecker.getConfiguration().setBoolean(Configuration.SPELL_IGNORESENTENCECAPITALIZATION, false);
    	spellChecker.getConfiguration().setBoolean(Configuration.SPELL_IGNOREMIXEDCASE, false);
    	spellChecker.getConfiguration().setInteger(Configuration.SPELL_THRESHOLD, searchDepth);
    }






      /***************************************************************************
     * @param text a string of text to check the spelling of
     * @return a hashtable of string[] suggestions keyed by the misspelled word
     **************************************************************************/
    public MisspelledCollection checkSpelling(String text)
		throws Exception
    {
    	misspelled = new MisspelledCollection();
    	int result = spellChecker.checkSpelling(new StringWordTokenizer(text));
    	if (result == SpellChecker.SPELLCHECK_CANCEL) {
    		throw new Exception("SpellChecker was terminated");
    	}
    	if (isEvaluationMode) {
    		misspelled.add("30DAYTRAIL", new String[]{"PLEASE","VISIT","FOUNDEO.COM","TO","PURCHASE"});
    	}


        return misspelled;
    }

    public void addDictionary(String path) throws Exception {
    	spellChecker.addDictionary(new SpellDictionaryHashMap(new File(path)));
    }

    public void setUserDictionary(String path) throws Exception {
    	spellChecker.setUserDictionary(new SpellDictionaryHashMap(new File(path)));
    }


    public SpellChecker getSpellChecker() {
    	return spellChecker;
    }

    /***************************************************************************
     * returns a string based on the spelling output format (does not strip html)
     * @param text text to spell check
     * @param format javascript or wddx see SpellingOutput
     * @see SpellingOutput
     **************************************************************************/
    public String checkSpelling(String text, String format)
		throws Exception
    {
		byte[] textBytes = text.getBytes();
		String thisText = new String(textBytes, getCharSet());
        return SpellingOutput.getOutputByName( format, checkSpelling(thisText) );
    }



    /***************************************************************************
     * returns a string based on the spelling output format, and can strip HTML
     * @param text text to spell check
     * @param format javascript or wddx see SpellingOutput
     * @param stripHTML when true takes out html tags
     * @see SpellingOutput
     **************************************************************************/
    public String checkSpelling(String text, String format, boolean stripHTML)
		throws Exception
    {
		byte[] textBytes = text.getBytes();
		String thisText = new String(textBytes, getCharSet());
        if (stripHTML) {
            thisText = StaticUtil.stripTags(thisText);
        }
        return SpellingOutput.getOutputByName( format, checkSpelling(thisText) );
    }

    /***************************************************************************
     * @param word a single word that is in question
     * @return true if mispelled, false if spelled correct
     **************************************************************************/
    public boolean isSpelledWrong(String word)
    {
        return !spellChecker.isCorrect(word);
    }

    /***************************************************************************
     * @param word A single mispelled word to query suggestions
     * @return a String array of size protected int numSuggestions suggestions
     **************************************************************************/
    public String[] getSuggestions(String word)
		throws Exception
    {
        String[] suggestions = new String[numSuggestions];
        MisspelledCollection mc = checkSpelling(word);
        if (mc.size() > 0) {
        	suggestions = mc.getSuggestionsAt(0);
        }
        return suggestions;
    }


    /***************************************************************************
     * Add a word to a dictionary
     **************************************************************************/
    public void addWord(String path, String word)
        throws Exception
    {
    	synchronized (lock) {
    		//make writing to dictionary single threaded
	    	File f = new File(path);
	    	if (!f.exists()) {
	    		throw new Exception("The user dictionary file does not exist at the path sepcified");
	    	}
	    	if (!f.canWrite()) {
	    		throw new Exception("The user dictionary file is not writable, please check permissions and make sure that you do not have the file locked or open.");
	    	}
	    	SpellDictionaryHashMap d = new SpellDictionaryHashMap(f);
	    	d.addWord(word.toLowerCase());
    	}

    }

    /***************************************************************************
     * @param numSuggestions The number of suggestions the spell checker will
     *        generate
     **************************************************************************/
    public void setNumSuggestions(int numSuggestions)
    {
        this.numSuggestions = numSuggestions;
    }


    /***************************************************************************
     * @param d search depth used in suggestion finding 0=shallowest
     * the default is 150.
     **************************************************************************/
    public void setSearchDepth(int searchDepth)
    {
        this.searchDepth = searchDepth;
        spellChecker.getConfiguration().setInteger(Configuration.SPELL_THRESHOLD, searchDepth);
    }

    /***************************************************************************
     * @return The search depth used to find suggestions
     **************************************************************************/
    public int getSearchDepth()
    {
        return searchDepth;
    }




	/***************************************************************************
     * @return true if set charlimit
     **************************************************************************/
    public int getCharLimit()
    {
		int thelimit = charlimit;
        return thelimit;
    }

	/***************************************************************************
     * @return true if set charlimit
     **************************************************************************/
    public boolean setCharLimit(int charlimit)
    {
		this.charlimit = charlimit;
        return true;
    }

	/****************************************************************************
	* Get the character set used to encode strings
	* @return String Character Set
	****************************************************************************/
	public String getCharSet() {
		return this.characterSet;
	}

	/****************************************************************************
	* Set the character set used to encode strings
	* @param charSet a string representing the character set
	* @see http://java.sun.com/j2se/1.3/docs/api/java/lang/package-summary.html#charenc
	*****************************************************************************/
	public void setCharSet(String charSet) {
		this.characterSet = charSet;
	}

	/**
	 * Ignore trailing S on a word if matched without the s or apos s
	 */
	public boolean getIgnoreTrailingS() {
		return this.ignoreTrailingS;
	}


	public void setIgnoreTrailingS(boolean ignore) {
		this.ignoreTrailingS = ignore;
	}

	public void spellingError(SpellCheckEvent event) {
		String word = event.getInvalidWord();
		int wordLen = word.length();
		if (wordLen <= this.minWordLength) {
			return;
		}
		if (ignoreTrailingS) {
			//allow words that end with s to be matched
			if (wordLen > 2 && word.charAt(wordLen-1) == 's') {
				if (word.charAt(wordLen-2) == '\'') {
					if (spellChecker.isCorrect(word.substring(0, wordLen-3))) {
						return;
					}
				} else if (spellChecker.isCorrect(word.substring(0, wordLen-2))) {
					return;
				}
			}
		}


		boolean capStart = word.charAt(0) <= 90;
		String[] sugs = new String[this.numSuggestions];
    	Iterator iter = event.getSuggestions().iterator();
    	int i = 0;
    	while (iter.hasNext() && i < this.numSuggestions) {
    		String s = ((Word)iter.next()).getWord();
    		if (capStart && s.charAt(0) <= 90) {
    			StringBuffer c = new StringBuffer();
    			c.append(s.charAt(0));
    			s.replaceFirst(c.toString(), c.toString().toUpperCase());
    		}
    		sugs[i++] = s;
    	}
    	if (i == 0) {
    		//no suggestions for the word
    		sugs = new String[this.numSuggestions];
    		sugs[0] = "";
    	}
    	for (;i<this.numSuggestions;i++) {
    		sugs[i] = "";
    	}

		misspelled.add(event.getInvalidWord(), sugs);
	}

	public String getVersionString() {
		StringBuffer sb = new StringBuffer();
		sb.append(getMajorVersion());
		sb.append('.');
		sb.append(getMinorVersion());
		sb.append('.');
		//sb.append("%BUILD_NUMBER%");
		sb.append("201507291946");
		return sb.toString();
	}

	public int getMajorVersion() {
		return 3;
	}

	public int getMinorVersion() {
		return 0;
	}

	public boolean isEvaluationMode() {
		return isEvaluationMode;
	}


}
