package com.foundeo.spellchecker;

/* ******************************************************************************
 * Author           Peter Freitag 
 * description      Misspelled Words, and suggestions Container Class
 ******************************************************************************/

import java.util.*;


/*******************************************************************************
 * A MisspelledCollection Object keeps a collection of misspelled words and
 * suggestions associated with them.
*******************************************************************************/
public class MisspelledCollection
{

    private Vector misspelled = new Vector();
    private Vector suggestions = new Vector();
    
	/*****************************************************************
	* Add misspelled word and suggestion to the collection
	* @param misspelledWord The word that is misspelled  
	* @param suggestions An array of suggestions for the misspelled word
	******************************************************************/ 
    public void add(String misspelledWord, String[] suggestions) {
        misspelled.addElement(misspelledWord);
        this.suggestions.addElement(suggestions);
    }
    
    /*****************************************************************
	* Add misspelled word and suggestion to the collection
	* @param misspelledWord The word that is misspelled  
	* @param suggestions An array of suggestions for the misspelled word
	******************************************************************/ 
    public void add(String misspelledWord, List suggestions) {
    	String[] sugs = new String[suggestions.size()];
    	Iterator iter = suggestions.iterator();
    	int i = 0;
    	while (iter.hasNext()) {
    		sugs[i++] = (String)iter.next();
    	}
    	add(misspelledWord, sugs);
    }
    /*****************************************************************
	* Get a misspelled word at specified index
	* @param i Position of the misspelled word
	* @return Misspelled word at index i
	******************************************************************/ 
    public String getWordAt(int i)
    {
        return (String)misspelled.elementAt(i);
    }
    
	/*****************************************************************
	* Get all suggestions for the word at the specified index
	* @param i Position of the misspelled word
	* @return An array of suggestions for the misspelled word at index i
	******************************************************************/ 
    public String[] getSuggestionsAt(int i)
    {
        return (String[])suggestions.elementAt(i);
    }
    
    /******************************************************************
	* Get the number of the misspelled words
	* @return The number of misspelled words
	*******************************************************************/
	public int size()
    {
        return misspelled.size();
    }
 
}


