package com.foundeo.spellchecker;

/* ******************************************************************************
 * Author           Peter Freitag
 * description      Generic Static Utility functions
 * Version:         1.0
 * License:			GPL
 ******************************************************************************/

/*******************************************************************************
 * A set of generic static utility functions.
 ******************************************************************************/
public abstract class StaticUtil
{
    /***************************************************************************
     * @param str A string to check and see if it is boolean, true unless
     *      explicitly specified to be false
     * @return a boolean evaluation of the string
     **************************************************************************/
    public static final boolean stringToBoolean(String str)
    {
        str = str.toLowerCase();
        if (str.equals("false")) {
            return false;
        }
        if (str.equals("no")) {
            return false;
        }
        if (str.equals("0")) {
            return false;
        }
        //if it's non false, it's true by definition
        return true;
    }

    /***************************************************************************
     * Removes <TAG> based Markup Languages from a document, also calls
     * stripInnerTags to remove whats inside script and style tag blocks.
     *
     * @param str Content to strip markup
     * @return Content without markup tags
     **************************************************************************/
    public static final String stripTags(String str)
    {
        String[] tags = {"script", "style"};
        str = stripInnerTags(str, tags);
        StringBuffer buffer = new StringBuffer(str.length());
        int startPos = str.indexOf("<");
        while (startPos >= 0)
        {
            int endPos = str.indexOf(">", startPos);
            if (endPos == -1) { break; }
            buffer.append(str.substring(0, startPos));
            buffer.append(" "); //add whitespace to prevent concatination
            buffer.append(str.substring(endPos+1, str.length()));

            str = buffer.toString();
            buffer.setLength(0);
            startPos = str.indexOf("<");
        }
        //PAF Added 7/29/2015 Hi&nbsp;Bill turns into HiBill
        str = str.replaceAll("&nbsp;","     ");
        str = str.replaceAll("&amp;"," &   ");
        //PAF Added 7/21/2010 to strip HTML entities
        str = str.replaceAll("&#?[0-9a-zA-Z-]{2,10};", " ");
        return str;
    }

    /***************************************************************************
     * Get's rid of what's inside a block tag, for instance if you are stripping
     * HTML you also want to remove what's inside of <script></script> tags.
     *
     * @param str Content to be formatted
     * @param tags An array of tag names, should be in lowercase!
     * @return A string without specified block tags
     **************************************************************************/
    public static final String stripInnerTags(String str, String[] tags)
    {
        StringBuffer buffer = new StringBuffer(str.length());
        buffer.append(" ");
        buffer.append(str);
        str = buffer.toString();
        String lcString = str.toLowerCase();
        //empty the buffer
        buffer.setLength(0);
        for (int i=0;i<tags.length;i++)
        {
            int startPos = lcString.indexOf("<" + tags[i]);
            while(startPos >=0)
            {
                int endPos = lcString.indexOf("</" + tags[i]);
                if (endPos > 0) //cant be zero
                {
                    buffer.append(str.substring(0, startPos));
                    buffer.append(" "); //add whitespace to prevent concatination
                    buffer.append(str.substring(endPos +
                        tags[i].length()+3, str.length()));

                    str = buffer.toString();
                    //empty the buffer
                    buffer.setLength(0);
                }
                else { break; }
                startPos = lcString.indexOf("<" + tags[i]);
            }
        }
        return str;
    }

	public static final String replace(String source, String find, String replace) {
		int pos = source.indexOf(find);
		if (pos != -1) {
			StringBuffer sb = new StringBuffer(source.length() + replace.length());
			sb.append( source.substring(0, pos) );
			sb.append( replace );
			sb.append( source.substring(pos+find.length()) );
			return sb.toString();
		}
		return source;
	}

	public static final String replaceAll(String source, String find, String replace) {
		while (source.indexOf(find) != -1) {
			source = replace(source, find, replace);
		}
		return source;
	}

}
